# base for measure views

from analyses.analysisbase import forms as app_forms
from analyses.analysisbase import tables as app_tables
from analyses.analysisbase.views.scenario import localnavigationlist
from analyses.analysisbase.views import helpers
from analyses.main import analyses_navigationlist

from django.contrib import messages
from django.core.urlresolvers import reverse
from django.db import models
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _

from installations.views.main import installations_navigationlist

from utils import crud, lists
from utils import forms as utils_forms
from utils import permissions
from utils.autocomplete import autocomplete
from utils import strings
from utils.versioning import versionedcrud
from utils.versioning.session import get_current_version
from utils.versioning.models import WorkVersion

class MixIn(object):
    "Common items for object views"
    include_attachments = True
    include_links = True
##    model = app_models.DegradationMeasure

    tabs = [
        {'name': 'description',
            'label': _('Description')},
        {'name': 'attentionpoints',
            'label': _('Attention points')},
        {'name': 'usage',
            'label': _('Usage')},
    ]

class display_base(MixIn, versionedcrud.DisplayObjectView):
    allow_replace = False
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        extra_context['navigationlist'] =  installations_navigationlist()
        extra_context['currentanalysistype'] = self.appconfig.analysistype_short
        if self.allow_replace and object.may_replace(self.request.user):
            extra_context['defaultbuttons'].append({'url': object.get_replace_url(), 'label': _('Replace')})

class display_description(display_base):
    form_class = None
    allow_replace = True

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.MeasureSheet, model=self.model)

class display_attentionpoints(display_base):
    allow_change = False
    allow_delete = False
    form_class = None
    table_class = None

    def add_to_extra_context(self, extra_context, object):
        super(display_attentionpoints, self).add_to_extra_context(extra_context, object)
        queryset = object.attentionpoints.all()
        if self.table_class:
            table = self.table_class(queryset)
        else:
            table = utils_forms.create_class_with_meta(app_tables.AttentionPointsTable, model=self.appconfig.get_model('attentionpoint'))(queryset)
        if self.appconfig.get_model('attentionpoint').may_add(self.request.user, parent=object):
            table.buttons = [{
                    'url': object.get_add_attentionpoint_url(),
                    'label': _('Add'),
                },
                {
                    'url': object.get_add_attentionpoint_from_suggestionlist_url(),
                    'label': _('Add from SL'),
                }]
        extra_context['table'] = table

class display_usage(display_base):
    allow_change = False
    allow_delete = False
    form_class = None
    table_class = None

    def add_to_extra_context(self, extra_context, object):
        super(display_usage, self).add_to_extra_context(extra_context, object)
        queryset = object.protectionlayers.all()
        if self.table_class:
            table = self.table_class(queryset)
        else:
            table = utils_forms.create_class_with_meta(app_tables.MeasureUsageTable, model=self.appconfig.get_model('protectionlayer'))(queryset)
        extra_context['table'] = table

##class display(MixIn, crud.TabContainer):
##    tabviews = {
##        'description': display_description(),
##        'attentionpoints': display_attentionpoints(),
##        'usage': display_usage(),
##    }
##    default = 'description'

class edit(MixIn, versionedcrud.EditObjectView):
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.MeasureForm, model=self.model)

class delete(MixIn, versionedcrud.DeleteObjectView):
    tabs = []
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.MeasureSheet, model=self.model)

    def get_default_url(self, object):
        return reverse('%s_measures' % self.appconfig.namespace)

class report(MixIn, versionedcrud.ReportObjectView):
    form_class = utils_forms.ReportForm

class add_to_protectionlayer(MixIn, versionedcrud.AddObjectView):
    tabs = []
    form_class = None
    linkform_class = None
    template = 'analysisbase/measure_add.html'

    def setup_from_args(self, object_id=None, *args, **kwargs):
        (self.version, self.specialversion) = get_current_version(self.request)
        self.protectionlayer = versionedcrud.get_versionedobject_or_404(self.appconfig.get_model('protectionlayer'), self.version, v_id = object_id)
        self.SL = bool(self.protectionlayer.get_treeelements()[0].scenario.suggestionlist)
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.MeasureForm, model=self.model)
        if not self.linkform_class:
            self.linkform_class = utils_forms.create_class_with_meta(app_forms.LinkMeasureForm, model=self.model)

    def save_instance(self, object):
        object.SL = self.SL
        object.save(self.request.user)
        # link to the pl
        self.appconfig.get_model('protectionlayer_measure')(protectionlayer=self.protectionlayer, measure=object).save(self.request.user)
        return object

    def get_default_url(self, object):
        return self.protectionlayer.get_absolute_url()

    def process_request(self):
        from utils.crud.views import RefererForm
        # special handling for second form
        self.errormessages = []
        if self.request.method == 'POST':
            preview = False
            self.refform = RefererForm(data=self.request.POST, prefix='management')
            self.form = self.get_form(self.object)
            self.linkform = self.linkform_class(prefix='link', data=self.request.POST, files=self.request.FILES, SL=self.SL)
            if self.request.POST.get('link',''):
                if self.linkform.is_valid():
                    # watch out: we get a real id from the form:
                    m_pk = self.linkform.cleaned_data.get('measure')
                    if m_pk:
                        measure = get_object_or_404(self.model, pk=m_pk)
                        if not self.appconfig.get_model('protectionlayer_measure').objects.filter(protectionlayer=self.protectionlayer, measure=measure, version__isnull=True).count():
                            newlink = self.appconfig.get_model('protectionlayer_measure')()
                            newlink.protectionlayer=self.protectionlayer
                            newlink.measure=measure
                            newlink.save(self.request.user)
                        self.set_message(self.object)
                        return self.get_redirect(self.refform, self.object)
            else:
                if self.form.is_valid():
                    self.object = self.save_form(self.form)
                    self.after_save()
                    self.set_message(self.object)
                    return self.get_redirect(self.refform, self.object)
            self.errormessages = [self.get_errormessage()]
        else:
            self.refform = RefererForm(prefix='management')
            self.refform.initial['referer'] = self.get_referer(self.object)
            self.form = self.get_form(self.object)
            self.linkform = self.linkform_class(prefix='link', SL=self.SL)

    def add_to_extra_context(self, extra_context, object):
        extra_context['linkform'] = self.linkform

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = None
    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.MeasureSheet, model=self.model)
        super(compare, self).setup_from_args(*args, **kwargs)

class replace(MixIn, versionedcrud.EditObjectView):
    tabs = []
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = utils_forms.create_class_with_meta(app_forms.ReplaceMeasureForm, model=self.model)
        super(replace, self).setup_from_args(*args, **kwargs)

    def get_form(self, object, **formargs):
        # no modelform, so no instance, but needs SL
        if self.request.method == 'POST':
            form = self.form_class(data=self.request.POST, files=self.request.FILES, SL=object.SL, **formargs)
        else:
            form = self.form_class(SL=object.SL, initial=self.get_initial(object), **formargs)
        self.setup_form(form, object)
        return form

    def get_initial(self, object):
        """
        Find a best match pl to suggest as replacement
        """
        firstchar = object.name[0]
        items = list(self.model.objects.filter(SL=object.SL).exclude(pk=object.pk).filter(name__startswith=firstchar).get_versionedobjects(version=WorkVersion).order_by('name').values_list('name', flat=True))
        if not items:
            return {}
        items.append(object.name)
        items.sort()
        pos = items.index(object.name)
        if pos == 0:
            name = items[1]
        elif pos == len(items)-1:
            name = items[pos-1]
        else:
            pr = strings.count_ident(items[pos-1], object.name)
            ne = strings.count_ident(items[pos+1], object.name)
            if pr > ne:
                name = items[pos-1]
            else:
                name = items[pos+1]
        item = self.model.objects.filter(SL=object.SL).exclude(pk=object.pk).filter(name=name).get_versionedobjects(version=WorkVersion).order_by('type')[0]
        return {'measure': item.id}

    def save_form(self, form):
        # get the chosen object and point all references to that object
        self.object2 = versionedcrud.get_versionedobject_or_404(self.model, WorkVersion, id=form.cleaned_data.get('measure'))
        if self.object != self.object2:
            for item in self.object.protectionlayers.all():
                if not self.appconfig.get_model('protectionlayer_measure').objects.filter(protectionlayer=item, measure=self.object2, version__isnull=True).count():
                    links = self.appconfig.get_model('protectionlayer_measure').objects.filter(protectionlayer=item, measure=self.object, version__isnull=True)
                    for link in links:
                        link.measure=self.object2
                        link.save(self.request.user)
                else:
                    links = self.appconfig.get_model('protectionlayer_measure').objects.filter(protectionlayer=item, measure=self.object, version__isnull=True)
                    for link in links:
                        link.delete(self.request.user)
            self.object.delete(self.request.user)
        return self.object

    def get_redirect(self, refform, object):
        if self.object2:
            return HttpResponseRedirect(self.object2.get_absolute_url())
        return HttpResponseRedirect('/')

    def set_message(self, object):
        message = self.message
        if not message:
            message = u'%s: %s %s' % (capfirst(_('replaced')), object._meta.verbose_name, object,)
        messages.info(self.request, message)

    def get_title(self, object):
        return u'%s: %s %s' % (capfirst(_('replace')), self.model._meta.verbose_name, object)


def type_lookup(request, appconfig):
    model = appconfig.get_model('measure')
    queryset = model.objects.filter(SL=False).values('type').order_by('type').distinct()
    return autocomplete(request, queryset, 'type', 'type')

def name_lookup(request, appconfig):
    model = appconfig.get_model('measure')
    (version, special) = get_current_version(request)
    queryset = model.objects.filter(SL=False).get_versionedobjects(version=version).values('name').order_by('name')
    return autocomplete(request, queryset, 'name', 'name')

def measure_extra_filter(queryset, data, appconfig):
    """
    Implement the special 'linked' filter which requires a very special query
    Used in filter and listobject
    """
    if data.get('linked'):
        if queryset:
            version = queryset[0]._version
            val = {u'True': True, u'False': False, True: True, False: False, 1: True, 0: False, u'1': True, u'0': False}[data['linked']]
            if val:
                crit = '> 0'
            else:
                crit = '= 0'
            t1 = '%s_%s' % (appconfig.namespace, appconfig.get_model('protectionlayer_measure').__name__.lower())
            t2 = '%s_%s' % (appconfig.namespace, appconfig.get_model('measure').__name__.lower())
            if version == WorkVersion:
                qs = queryset.extra(
                    where = ['''
                        (SELECT COUNT(*) from "%s" X
                        WHERE X."measure_id" = "%s"."v_id"
                        AND NOT (X."deleted" = %%s)
                        AND X."version_id" is null
                        ) %s''' %
                        (t1, t2, crit)], params=[True,]
                    ).get_versionedobjects(version)
            else:
                datev = models.DateTimeField().get_db_prep_value(version.date)
                qs = queryset.extra(
                    where = ['''
                        (SELECT COUNT(*) FROM "%s" X LEFT OUTER JOIN "versioning_version" Y ON X."version_id"=Y."id"
                        WHERE X."measure_id" = "%s"."v_id"
                        AND NOT (X."deleted" = %%s)
                        AND Y."date" =
                            (SELECT MAX(C."date")
                                FROM "%s" B LEFT OUTER JOIN "versioning_version" C on B."version_id"=C."id"
                                WHERE X."v_id" = B."v_id" and C."date" <= %%s)
                        ) %s''' %
                        (t1, t2, t1, crit)], params=[True, datev,]
                    ).get_versionedobjects(version)
            return qs, True
    return queryset, False


class MeasureFilter(crud.Filter):
    form_class = None
##    model = app_models.DegradationMeasure
    filters = [
                {'filteritem': 'name', 'parameter': 'name__icontains', 'type': 'normal'},
                {'filteritem': 'tag', 'parameter': 'tag__icontains', 'type': 'normal'},
                {'filteritem': 'type', 'parameter': 'type', 'type': 'normal'},
            ]

    def extra_filter(self, queryset, data):
        return measure_extra_filter(queryset, data, self.appconfig)

    def get_form(self):
        if not self.form_class:
            return utils_forms.create_class_with_meta(app_forms.MeasureFilterForm, model=self.model)
        return self.form_class

class MeasureListObject(helpers.TableListObject):
    defaultsort = 'name'
    default_table_class = app_tables.MeasuresTable

    def get_title(self):
        return u'%s %s' % (_('Measures'), self.appconfig.analysistype)

    def get_query_set(self):
        return self.model.objects.exclude(SL=True).get_versionedobjects(self.version)

    def extra_filter(self, queryset, data):
        qs, filtered = measure_extra_filter(queryset, data, self.appconfig)
        return qs

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('measures'):
            for item in queryset:
                html_strings.append('<pdf:nextpage>')
                html_strings.extend(item.get_report_body(user, language, choices))

    def add_css(self, css_files, queryset, user, language, choices):
        if choices.get('measures'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class measures(helpers.TableListView):
    "List of measures"

##    filter = MeasureFilter()

    default_table_class = app_tables.MeasuresTable
    default_csv_table_class = app_tables.MeasuresCSVTable
    defaultsort = 'name'
    paginate_by = 100
    redirect_on_unique = True
    allow_pdf = True
    allow_csv = True
    pdfsendmail = True

    def get_title(self):
        return u'%s %s' % (_('Measures'), self.appconfig.analysistype)

    def get_query_set(self, request):
        return self.model.objects.exclude(SL=True).get_versionedobjects(self.version)

    def add_to_extra_context(self, extra_context):
        super(measures, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  analyses_navigationlist(self.appconfig.namespace)
        extra_context['tablist'] =  self.get_navigationlist(self.appconfig.namespace)
        extra_context['currenttab'] = 'measures'
        extra_context['breadcrumbs'] = [{'label': _('Lists'), 'url': u''}]

    def get_navigationlist(self, namespace):
        return localnavigationlist(namespace)


