from django.core.management.base import BaseCommand, CommandError
from django.db import transaction
import logging


class Command(BaseCommand):
    args = ''

    def handle(self, *labels, **options):
       # create logger
        logger = logging.getLogger()
        logger.setLevel(logging.DEBUG)
        # create console handler and set level to debug
        ch = logging.StreamHandler()
        verbosity = int(options.get('verbosity', 1))
        if verbosity == 0:
            ch.setLevel(logging.ERROR)
        elif verbosity == 2:
            ch.setLevel(logging.DEBUG)
        else:
            ch.setLevel(logging.INFO)
        formatter = logging.Formatter("%(levelname)s - %(message)s")
        # add formatter to ch
        ch.setFormatter(formatter)
        # add ch to logger
        logger.addHandler(ch)

        return rebuild_trees()

@transaction.commit_on_success
def rebuild_trees():
    from installations.models import SectionTreeElement
    from analyses.contacts.models import ContactTreeElement
    from analyses.degradations.models import DegradationTreeElement
    from analyses.deviations.models import CauseTreeElement
    from analyses.dispersions.models import DispersionTreeElement
    from analyses.explosionprevention.models import ExplosionPreventionTreeElement
    from analyses.explosionprotection.models import ExplosionProtectionTreeElement
    from analyses.fires.models import FireTreeElement
    from analyses.releases.models import ReleaseTreeElement

    logger = logging.getLogger()

    SectionTreeElement.tree.rebuild()
    logger.debug('rebuild installation trees')

    ContactTreeElement.tree.rebuild()
    logger.debug('rebuild contact scenario trees')

    DegradationTreeElement.tree.rebuild()
    logger.debug('rebuild degradation scenario trees')

    CauseTreeElement.tree.rebuild()
    logger.debug('rebuild deviation scenario trees')

    DispersionTreeElement.tree.rebuild()
    logger.debug('rebuild dispersion scenario trees')

    ExplosionPreventionTreeElement.tree.rebuild()
    logger.debug('rebuild explosion prevention scenario trees')

    ExplosionProtectionTreeElement.tree.rebuild()
    logger.debug('rebuild explosion protection scenario trees')

    FireTreeElement.tree.rebuild()
    logger.debug('rebuild fire scenario trees')

    ReleaseTreeElement.tree.rebuild()
    logger.debug('rebuild release scenario trees')

    return 'finished rebuilding trees'

